import io
import os

import click

from . import read
from ..test.ssl import get_hostnames

CONFIG_INI = """# -*- mode: conf -*-
[projects]
eggs_dir=~/.buildout/eggs
py2_exe=~/.virtualenvs/base2/bin/python
py3_exe=~/.virtualenvs/base3/bin/python3

[stats]
api_url = http://projects.thux.it/api
user = stats
password = demo

[sync]
password=
#sql_dump_dir = /var/tmp

"""


@click.command('edit')
@click.option('-c', '--create', is_flag=True, default=False,
              help="Create or modify a configuration")
@click.pass_context
def edit(ctx, create):
    """Edit main jmb configuration"""

    create_customization(ctx.obj)

def create_customization(conf=None, filename=None):

    if conf:
        if not filename:
            filename = conf.filename

    if not filename:
        filename = read.get_config_filename()
    if not os.path.exists(filename):
        content = click.edit(CONFIG_INI)
        if content is not None:
            if not os.path.exists(os.path.dirname(filename)):
                os.makedirs(os.path.dirname(filename))
            with io.open(filename, 'w') as f:
                f.write(content)
                click.secho("written file %s" % filename, fg='magenta')
    else:
        click.edit(filename=filename)

@click.command('certbot')
@click.option('-l', '--lets-encrypt', is_flag=True, help="Use lets's encrypt rather that certbot (Ubuntu)")
@click.option('--www/--no-www', is_flag=True, help="Test/Don't prepend www", default=True)
@click.option('--permutate', '-x', help="Permutate on all domains in string eg: -x 'it com org'")
@click.argument('domains', nargs=-1)
def certbot(lets_encrypt, www, permutate, domains):
    "Compose the row to issue certbot command"
    hostnames = get_hostnames(domains, permutate, www)

    cmd = lets_encrypt and 'letsencrypt' or 'certbot'
    cmd += " certonly --webroot -w /tmp -d" + " -d ".join(hostnames)
    click.secho(cmd, fg="cyan")
