import pyquery
import click
import requests
from requests.auth import HTTPBasicAuth


@click.command('domain')
@click.option('--ssl/--no-ssl', is_flag=True, help="Test/Don't test https", default=True)
@click.option('--www/--no-www', is_flag=True, help="Test/Don't prepend www", default=True)
@click.option('--user', '-u', help="The user for basic authentication")
@click.option('--password', '-p', help="The pass for basic authentication")
@click.option('--permutate', '-x', help="Permutate on all domains in string eg: -x 'it com org'")
@click.option('--verbose', '-v', help="Be verbose and declare the hostname you're testing")
@click.argument('domains', nargs=-1)
def test_response(ssl, www, user, password, domains, permutate, verbose):
    "Test response from servers"
    protocols = ['http']
    if ssl:
        protocols += ['https']
    hostnames = get_hostnames(domains, permutate, www)
    for hostname in hostnames:
        for proto in protocols:
            url = u"{proto}://{hostname}".format(**locals())
            test_url(url, user, password, verbose)


def get_hostnames(domains, permutate='', www=True):
    if isinstance(domains, basestring):
        domains = [domains]
    if www:
        hostnames =  list(domains) + ['www.%s' % domain for domain in domains]
    else:
        hostnames = domains
    if permutate:
        hostnames = ['{}.{}'.format(host, ext) for host in hostnames for ext in permutate.split()]
    return hostnames


def test_url(url, user=None, password=None, verbose=False):
    if verbose:
        click.secho("*** %s" % url, fg='magenta')
    if user:
        auth = HTTPBasicAuth(user, password)
    else:
        auth = None
    try:
        resp = requests.get(url, auth=auth, allow_redirects=False)
    except requests.exceptions.ConnectionError:
        click.secho(u"{}: connection FAILED".format(url), fg='red')
        return

    if resp.status_code >= 300 and resp.status_code < 400:
        Location = resp.headers['Location']
        msg = u"{url} => {resp.status_code}  ({Location})".format(**locals())
        click.secho(msg)
    elif resp.status_code >= 200 and resp.status_code < 300:
        d = pyquery.PyQuery(resp.content)
        try:
            title = (d('title').contents()[0]).strip()
        except IndexError:
            title = "NO TITLE"

        msg = u"{url} => {resp.status_code} ({title})".format(**locals())
        click.echo(click.style(msg, fg='green'))
    else:
        msg = u"{url} {resp}".format(url=url, resp=resp.json)
        click.echo(click.style(url, fg='red'))
